/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.DecodingException;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.opt4j.core.genotype.CompositeGenotype;
import org.opt4j.core.problem.Creator;

/**
 * {@link Creator} for {@link CompositeGenotype}s. The creation of {@link ComposableGenotype}s
 * comprising the {@link CompositeGenotype} must be handled by implementing classes.
 * 
 * @author diewald
 */
public interface CompositeCreator<G extends CompositeGenotype<Class<? extends ComposableGenotype>, ? extends ComposableGenotype>>
		extends Creator<G> {

	/** Creates the {@link ComposableGenotype}. */
	G createComposite();

	/**
	 * Calls the {@code create()} method of {@link ComposableGenotype}s registered at this
	 * {@link CompositeCreator}.
	 * 
	 * @throws DecodingException
	 */
	<N extends ComposableGenotype, P extends Phenotype> void createSubGenotypes(G compositeGenotype)
			throws DecodingException;

	/**
	 * Registers a {@link Creator} of a {@link ComposableGenotype} with this
	 * {@link CompositeCreator}.
	 */
	<C extends ComposableGenotype> void registerSubCreator(ComposableCreator<?> subCreator);
}
