/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.CompositeDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.problem.CompositeProblemModuleBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.opt4j.core.genotype.CompositeGenotype;
import org.opt4j.core.problem.Evaluator;

/**
 * Base class for problems that are solved within this DSE. It allows to create
 * {@link EvaluatorWrapper}s which associate {@link ExplorationTarget}s with {@link Evaluator}s of
 * possible solutions to the problem that shall be solved by the DSE.
 * 
 * @author diewald
 */
public abstract class DseProblemModuleBase<C extends CompositeGenotype<Class<? extends ComposableGenotype>, ComposableGenotype>, P extends Phenotype, CD extends CompositeDecoder<C, P>>
		extends CompositeProblemModuleBase<C, P, CD> {

	/** DSE configuration */
	protected DseSpecification dse;

	/** Representation of input system */
	// TODO(#3270)
	protected SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Set of requested {@link Phenotype}s that form the solution set. */
	protected Set<Class<? extends Phenotype>> requestedSolutions;

	/**
	 * Constructor the DSE specification and an adapter to the input system model has to be
	 * specified.
	 */
	public DseProblemModuleBase(DseSpecification dse,
			SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter,
			Set<Class<? extends Phenotype>> requestedSolutions) {
		this.dse = dse;
		this.systemModelAdapter = systemModelAdapter;
		this.requestedSolutions = requestedSolutions;
	}
}
