/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.module.EvaluatorModule;
import org.fortiss.af3.exploration.alg.dse.module.IEvaluatorModule;
import org.fortiss.af3.exploration.dsl_v2.model.expression.IExpression;
import org.fortiss.af3.exploration.lang.ExplorationExpressionEvaluator;
import org.fortiss.af3.exploration.model.ExplorationTarget;

/**
 * Base class for {@link IEvaluatorModule}s of {@link IExpression}s. It holds a reference to the
 * expression to be evaluated and its parser.
 * 
 * @author diewald
 */
public abstract class ExpressionEvaluatorBase<R, T extends ExplorationTarget<R>, P extends Phenotype>
		extends EvaluatorModule<R, T, P> {

	/** DSML expression defining the evaluation to be performed. */
	protected IExpression expression;

	/** References the parser to be applied on the referenced {@link IExpression}. */
	protected ExplorationExpressionEvaluator expressionParser;

	/** Prototype Constructor. */
	protected ExpressionEvaluatorBase(Class<T> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Creates an evaluator for a given exploration goal. */
	protected ExpressionEvaluatorBase(T explorationTarget) {
		super(explorationTarget);
		this.expression = explorationTarget.getExpression();
	}
}
