/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.EvaluatorInjectorProvider;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.evaluator.BooleanValue;
import org.fortiss.af3.exploration.alg.dse.evaluator.GenericValue;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.ComponentMultiAllocationConstraint;
import org.opt4j.core.Value;

import com.google.common.collect.ClassToInstanceMap;
import com.google.inject.Provides;
import com.google.inject.name.Named;

/**
 * Deployment constraint which defines the allowed deployments of {@link ITaskAdapter}s to
 * {@link IExecutionUnitAdapter}s. All other allocations are
 * forbidden.
 * 
 * @author huang
 */
// TODO: Register this evaluator for AstractTaskMappings.
public class ComponentMultiAllocationConstraintEvaluator<S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
		extends ConstraintEvaluatorBase<Boolean, ComponentMultiAllocationConstraint, T> {

	/** Constructor. */
	public ComponentMultiAllocationConstraintEvaluator(
			Class<ComponentMultiAllocationConstraint> explorationTargetType) throws Exception {
		super(explorationTargetType);
	}

	/** Constructor. */
	public ComponentMultiAllocationConstraintEvaluator(
			ComponentMultiAllocationConstraint explorationTarget) throws Exception {
		super(explorationTarget);
	}

	/** {@inheritDoc} */
	@Provides
	@Named("ComponentMultiAllocationConstraintEvaluator")
	public GenericValue<Boolean> evaluateGoal(T phenotype,
			FlatPhenotypeMap<Phenotype> phenotypeTypeMap,
			ClassToInstanceMap<EObject> transformedModels) {
		for(InstantiatedTaskMappingEntry mappingEntry : phenotype.getMappingEntries()) {
			if(explorationTarget.getAllRightMultiLocationModelElements()
					.contains(mappingEntry.getSource().getObject())) {
				return new BooleanValue(false);
			}
		}
		return new BooleanValue(true);
	}

	/** {@inheritDoc} */
	@Override
	public boolean validate(ExplorationConstraint<?> expTarget,
			EvaluatorInjectorProvider evalInjProvider, Value<?> value) {
		Component constrComp = ((ComponentMultiAllocationConstraint)expTarget).getComponent();
		InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry> itmEnc =
				evalInjProvider.getInjector().getInstance(InstantiatedTaskMappingEncoding.class);

		Collection<InstantiatedTaskMappingEntry> entries = new ArrayList<>();
		for(InstantiatedTaskMappingEntry mappingEntry : itmEnc.getMappingEntries()) {
			ITaskAdapter<?> compAdp = mappingEntry.getSource();
			if(compAdp.getObject() == constrComp) {
				entries.add(mappingEntry);
			} else if(compAdp.isInstantiation() &&
					compAdp.getReplacedComponent().getObject() == constrComp) {
				entries.add(mappingEntry);
			}
		}

		if(entries.isEmpty()) {
			return false;
		}

		for(IMappingEntry<?, ?> entry : entries) {
			if(!((ComponentMultiAllocationConstraint)expTarget).getExecutionUnits()
					.contains(entry.getTarget().getObject())) {
				return false;
			}
		}
		return true;
	}
}
