/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.af3.exploration.alg.util.AF3Utils.getAnnotationChecked;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.exception.InvalidModelException;
import org.fortiss.af3.platform.hierarchic.model.annotation.FailureRate;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.IVirtualizationPlatformArchitectureElement;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Base adapter for concrete system model elements derived from {@link IResourceAdapter}.
 * 
 * @author barner
 */
// TODO: Use extends IResource, here? Same would apply to Requester/resource.
public class AF3ResourceAdapter<T extends IModelElement> extends AF3ModelElementAdapter<T>
		implements IResourceAdapter<T> {
	/** Failure rate annotation */
	protected Double failureRate = 0.0;

	/**
	 * Constructor. Creates an {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 * 
	 * @throws InvalidModelException
	 *             if a required annotation cannot be found.
	 */
	public AF3ResourceAdapter(T modelElement) throws InvalidModelException {
		super(modelElement);
		if(!isVirtual()) {
			failureRate = getAnnotationChecked(getObject(), FailureRate.class).getFailureRate_FIT();
		}
	}

	/** {@inheritDoc} */
	@Override
	public Double getFailureRate() {
		return failureRate;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isVirtual() {
		return getObject() instanceof IVirtualizationPlatformArchitectureElement;
	}
}
