/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.moea.model.predefined.impl;

import static java.util.Collections.emptyList;

import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.fortiss.af3.exploration.moea.model.predefined.MultiLocationConstraint;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Static method implementations for {@link MultiLocationConstraint}s.
 * 
 * @author diewald
 */
public class MultiLocationConstraintStaticImpl {

	/**
	 * Static implementation of {@link MultiLocationConstraint#getLeftMultiLocationModelElement}.
	 */
	public static <LHT extends IModelElement, RHT extends IModelElement> LHT
			getLeftMultiLocationModelElement(MultiLocationConstraint<LHT, RHT> constr) {
		if(constr == null) {
			return null;
		}
		if(!constr.getSingleLocationConstraints().isEmpty()) {
			// Get(0): The left hand side is equal for all location constraints, so the first one
			// suffices.
			return constr.getSingleLocationConstraints().get(0).getLeftModelElement();
		}
		return null;
	}

	/**
	 * Static implementation of
	 * {@link MultiLocationConstraint#getAllRightMultiLocationModelElements}.
	 */
	@SuppressWarnings("unchecked")
	public static <LHT extends IModelElement, RHT extends IModelElement> EList<RHT>
			getAllRightMultiLocationModelElements(MultiLocationConstraint<LHT, RHT> constr) {
		if(constr == null) {
			return null;
		}
		if(!constr.getSingleLocationConstraints().isEmpty()) {
			List<RHT> cList = constr.getSingleLocationConstraints().parallelStream()
					.map(c -> c.getRightModelElement()).collect(Collectors.toList());
			BasicEList<RHT> reList = new BasicEList<>();
			reList.addAll(cList);
			return reList;
		}
		return (EList<RHT>)emptyList();
	}
}
